note
   description: "A generic ITERATOR for an indexed COLLECTION"
   date: "2011 April 22"
   version: "1.0"
   author: "Michel Train & Gunnar Gotshalks"

class COLLECTION_ITERATOR[E]
  inherit ITERATOR2[E]

create make

--------------------------------------------------------------------------------
feature {NONE} -- Creation

make (target: like target_collection)
    -- Creation and initialization
  require target_not_void: target /= Void
  do
    target_collection := target
  end

--------------------------------------------------------------------------------
feature -- Queries

exhausted, is_done : BOOLEAN
    -- Whether all elements have been visited
  do
    target_collection.finish
    Result:= index > target_collection.index
  end

item : E
    -- The current element in the iteration
  do
    Result := target_collection.i_th(index)
  end

--------------------------------------------------------------------------------
feature -- Commands

start, first
    -- Go to the aggregate first position: start the iteration
  do
    target_collection.start
    index := target_collection.index
  end

next
    -- Advance to the next element
  do
    index := index + 1
  end

--------------------------------------------------------------------------------
feature {NONE} -- Private

index : INTEGER -- The cursor on the target collection

target_collection : LINKED_LIST[E] -- The target collection itself

aggregate_state : INTEGER
    -- Encoding of the aggregate abstract state that should not
    -- change if the Iterator is not robust
  do
    Result := target_collection.count -- Gross simplification
  end

invariant
   target_collection_not_void: target_collection /= Void

end
