note
   description: "Definition of the ITERATOR interface.%
     % The Intent of the iterator is to provide a way to access the elements%
     % of an aggregate object sequentially without exposing its underlying%
     % representation."
   date: "2011 April 22"
   version: "1.0"
   author: "Michel Train & Gunnar Gotshalks"

deferred class ITERATOR2[E]

--------------------------------------------------------------------------------
feature -- Queries

exhausted, is_done : BOOLEAN
    -- Whether all elements have been visited
  deferred
  end

item : E
    -- The current element in the iteration
  require not_exhausted: not is_done
    deferred
  ensure
     -- If not robust, we cannot change the aggregate while traversing it
     robustness: is_robust or else aggregate_state = old(aggregate_state)
  end

is_robust : BOOLEAN
    -- Whether we can insert and delete items in the collection
    -- without interfering with the traversal
  do -- False by default
  end

--------------------------------------------------------------------------------
feature -- Commands

start, first
    -- Go to the aggregate first position: start the iteration
  deferred
  end

next
    -- Advance to the next element
  require not_exhausted: not is_done
    deferred
  end

--------------------------------------------------------------------------------
feature {NONE} -- Private

aggregate_state : INTEGER
    -- Encoding of the aggregate abstract state that should not
    -- change if the Iterator is not robust
  deferred
  end

end

